#include <iostream>
#include <queue>

// Ako imamo neparan broj elemenata, medijana mora da bude minimalni element "desnog" hipa, tj
// minimalni element u hipu gde su svi elementi veci ili jednaki medijani.
// Ako imamo paran broj elemenata, medijana je artimeticka sredina minimalnog elementa desnog hipa i
// maksimalnog elementa levog hipa
double medijana(const auto &manji_od_sredine, const auto &veci_od_sredine)
{
    if(manji_od_sredine.size() == veci_od_sredine.size())
        return (manji_od_sredine.top() + veci_od_sredine.top()) / 2.0;
    return veci_od_sredine.top();
}

// U desnom hipu cuvamo sve elemente koji su jednaki ili veci od medijane (ovo ce biti min heap, pa ce medijana biti bas minimalni
// element ovog hipa). U levom hipu se nalaze elementi koji su manji od medijane. Pokusavamo da oba hipa imaju jednak broj elemenata.
void dodaj_element(auto &manji_od_sredine, auto &veci_od_sredine, int x)
{
    // Na pocetku kada su oba hipa prazna, dodajemo element u desni hip koji moze da ima jedan element vise (samu medijanu)
    if(veci_od_sredine.empty())
        veci_od_sredine.push(x);
    else {
        // Ako je element manji ili jednak medijani (trenutnom minimalnom elementu na desnom hipu), element dodajemo u levi hip
        if(x <= veci_od_sredine.top())
            manji_od_sredine.push(x);
        // U suprotnom (element je veci od medijane) dodajemo ga u desni hip
        else
            veci_od_sredine.push(x);

        // Levi hip ne sme da ima vise elemenata od desnog (ako imamo neparno elemenata, medijana je u desnom hipu, a ako imamo
        // jednak broj elemenata, onda oba hipa imaju jednako elemenata i medijana je aritmeticka sredina korena oba hipa)
        // Ako levi hip ima vise elemenata, onda prebacujemo maksimalni element u desni hip
        if(manji_od_sredine.size() > veci_od_sredine.size()) {
            veci_od_sredine.push(manji_od_sredine.top());
            manji_od_sredine.pop();
        }
        // Desni hip sme da ima tacno jedan element vise od levog. Ako se desi da imamo 2 elementa vise, minimalni element
        // (medijanu) prebacujemo u levi hip i time dobijamo da oba hipa imaju jednako elemenata
        else if(veci_od_sredine.size() > manji_od_sredine.size() + 1) {
            manji_od_sredine.push(veci_od_sredine.top());
            veci_od_sredine.pop();
        }
    }
}

int main ()
{
    std::priority_queue<int, std::vector<int>, std::greater<int>> veci_od_sredine;
    std::priority_queue<int, std::vector<int>, std::less<int>> manji_od_sredine;

    int x;
    char c;

    while (1) {
        std::cin >> c;
        if (c == 'k')
            return 0;
        else if (c == 'm')
            std::cout << medijana(manji_od_sredine, veci_od_sredine) << "\n";
        else {
            std::cin >> x;
            dodaj_element(manji_od_sredine, veci_od_sredine, x);
        }
    }

    return 0;
}