#include <iostream>
#include <stack>
#include <string>
#include <vector>

// Funkcija koja bi trebalo da parsira nisku po " "
std::vector<std::string> parse_string(const std::string &expression)
{
    // Posto saljemo const referencu moramo da uzmemo kopiju niske, ne zelimo originalni izraz da menjamo
    std::string s = expression;
    // Pravimo vektor niski koje predstavljaju delove izraza
    std::vector<std::string> parts;

    int pos = 0;

    // Dok god nailazimo na " " (jedan space) hocemo da se krecemo kroz nasu nisku
    // std::string::npos govori da smo dosli do kraja niske, i metod find vraca bas tu predefinisanu vrednost ukoliko ne nadje trazeni karakter (nisku u nasem slucaju)
    while ((pos = s.find(" ")) != std::string::npos) {
        // uzimamo deo niske od pocetka do pojave space-a
        parts.push_back(s.substr(0, pos));
        // Iz niske s brisemo sve karaktere od pocetka do jedne pozicije iza " ". Sta dobijamo na ovaj nacin? Ako je izraz bio 12 4 + kada nadjemo " " na poziciji 2, mi brisemo sve karaktere od 0 do 3 i dobijamo nisku 4 +
        // Nakon njenog parsiranja na isti nacin dobicemo samo +
        s.erase(0, pos + 1);
    }

    // Na kraju ce pos da bude kraj niske, pa hocemo sve karaktere od pocetka do kraja niske da dodamo u vektor. Recimo da je niska samo +, dodajemo + u vektor
    parts.push_back(s.substr(0, pos));

    // Vracamo vektor "delova" niske. U sustini dobijamo operacije sa kojima radimo i brojeve
    return parts;
}

int main ()
{
    // Koristimo stek stringova. Zasto ne karakteri? Ako idemo samo karakter po karakter onda tesko mozemo da imamo visecifrene brojeve
    std::stack<std::string> s;

    // 3 4 + 2 * 7 /
    std::string expression = "5 7 + 6 2 - *";

    // Promenljive koje ce biti operandi za operacije koje primenjujemo
    int operand_1, operand_2;

    // Uzimamo sve "delove" izraza koji nisu beline
    std::vector<std::string>parts = parse_string(expression);

    // Prolazimo kroz "delove". Zasto referenca? std::string nije primitivni tip (int, float, bool...) i onda je dobra praksa da saljemo po referenci zbog efikasnosti
    for (const auto &part : parts) {
        // Ukoliko smo naisli na +, treba da uzmemo poslednje 2 vrednosti sa steka, saberemo ih i vratimo na tek novu vrednost jer ako imamo izraz 12 5 + 6 +, treba da sracunamo 12 + 5 = 17 i su operandi drugog + 17 i 6, pa nam
        // i 17 treba na steku
        if (part == "+") {
            // Uzimamo operand 1
            // Funkcija stoi string pretvara u integer, i mi hocemo element sa vrha steka
            operand_1 = std::stoi(s.top());
            // Skidamo element sa steka
            s.pop();
            // Uzimamo operand 1
            // Funkcija stoi string pretvara u integer, i mi hocemo element sa vrha steka
            operand_2 = std::stoi(s.top());
            // Skidamo element sa steka
            s.pop();

            // Na vrh steka dodajemo rezultat sabiranja operanada 1 i 2, posto stek radi sa stringovima koristimo funkciju to_string da prebacimo integer u string
            s.push(std::to_string(operand_1 + operand_2));
        }
        else if (part == "*") {
            // Uzimamo operand 1
            // Funkcija stoi string pretvara u integer, i mi hocemo element sa vrha steka
            operand_1 = std::stoi(s.top());
            // Skidamo element sa steka
            s.pop();
            // Uzimamo operand 1
            // Funkcija stoi string pretvara u integer, i mi hocemo element sa vrha steka
            operand_2 = std::stoi(s.top());
            // Skidamo element sa steka
            s.pop();

            // Na vrh steka dodajemo rezultat mnozenja operanada 1 i 2, posto stek radi sa stringovima koristimo funkciju to_string da prebacimo integer u string
            s.push(std::to_string(operand_1 * operand_2));
        }
        else if (part == "-") {
            // Kod oduzimanja uzimamo prvo drugi operand jer ako pise 7 2 - to znaci 7 - 2, ako redom smestamo elemente na stek imacemo 7 pa 2 i zato je operand 1 7 a operand 2 je 2.
            // Druga varijanta je da racunamo operand_2 - operand_1. Ispravno je i jedno i drugo.
            operand_2 = std::stoi(s.top());
            s.pop();
            operand_1 = std::stoi(s.top());
            s.pop();
            s.push(std::to_string(operand_1 - operand_2));
        }
        else if (part == "/") {
            // Kod deljenja uzimamo prvo drugi operand jer ako pise 14 7 / to znaci 14 / 7, ako redom smestamo elemente na stek imacemo 14 pa 7 i zato je operand 1 14 a operand 2 je 7.
            // Druga varijanta je da racunamo operand_2 / operand_1. Ispravno je i jedno i drugo.
            operand_2 = std::stoi(s.top());
            s.pop();
            operand_1 = std::stoi(s.top());
            s.pop();
            if (operand_2 == 0) {
                std::err << "Deljenje 0!\n";
                return 1;
            }
            s.push(std::to_string(operand_1 / operand_2));
        }
        // Ako nije u pitanju operacija, imamo neki broj i samo ga dodajemo na stek
        else {
            s.push(part);
        }
    }

    // Na kraju je jedina vrednost na steku bas vrednost izraza pa je ispisujemo
    std::cout << s.top() << std::endl;

    return 0;
}
